import {
	FTable,
	SelectableRows,
	Row,
	ColumnWidths,
	ColumnWidthsLayoutMode,
	ResizableColumns,
	ResizableColumnsEvent,
	Scrollable,
	Filterable,
	MaxRows,
	Sortable,
	DraggableColumns,
	StickyRows
	/*, ExpandableRows*/
} from '@fathom-london/ftable-js';

require('./styles.css');
const demoData = require('./demo_data');

console.info('Running fTable-js dev-app');
console.info('SelectableRows', SelectableRows);

// Create data

const
	numRows = 200;//,
// columns = [
// 	{
// 		headerValue: 'Name',
// 		cellValue: '{{ $row.model.productName }}',
// 		id: 'name',
// 		width: 300
// 	},
// 	{
// 		headerValue: 'Price',
// 		cellValue: '{{"£" + $row.model.productPrice}}',
// 		id: 'price',
// 		width: 100,
// 		maxWidth: 150,
// 		minWidth: 70
// 	}
// ],
// rows = _getRows(numRows);

function _getRows(numRows) {
	const rows = [];
	for (let i = 0; i < numRows; i++) {
		rows.push({
			productName: `(${i + 1}) Product ${Math.floor(Math.random() * 5000)}`,
			productPrice: Math.floor(Math.random() * 500),
			id: i
		});
	}
	return rows;
}

const columns = demoData.demo_columns,
	rows = demoData.demo_data;


// Create fTable


// SELECTABLE
const selectableRows = new SelectableRows({multiSelect: true});
selectableRows.rangeSelect = true;
selectableRows.onRowSelected$.subscribe((row: Row) => console.log('dev-app: onRowSelected$:', row));
selectableRows.onRowDeselected$.subscribe((row: Row) => console.log('dev-app: onRowDeselected$:', row));
selectableRows.selectedRows$.subscribe((rows: Row[]) => console.log('dev-app: selectedRows$:', rows));

// COLUMN WIDTHS
const columnWidths = new ColumnWidths({layoutMode: ColumnWidthsLayoutMode.ABSOLUTE});

// RESIZABLE COLUMNS
const resizableColumns = new ResizableColumns();
resizableColumns.onResized$.subscribe((event: ResizableColumnsEvent) => {
	console.log('dev-app: resizableColumns.onResized$:', event);
});
resizableColumns.onResizing$.subscribe((event: ResizableColumnsEvent) => {
	console.log('dev-app: resizableColumns.onResizing$:', event);
});

// FILTERABLE
const filterable = new Filterable();

const input = document.querySelector('input[data-filter]');
input.addEventListener('input', (event) => {
	console.log('dev-app filter:', event.target.value);

	const filter = isExpression(event.target.value)
		? getFilters(event.target.value)
		: event.target.value;

	filterable.filters$.next(filter);
});

const presets = document.querySelectorAll('[data-filter-preset]');
for (let preset of presets) {
	console.log('dev-app:', preset);
	preset.addEventListener('click', (event) => setFilter(event.target.getAttribute('data-filter-preset')));
}

function setFilter(str) {
	input.value = str;
	filterable.filters$.next(getFilters(str));
}

function getFilters(search) {
	var expression = isExpression(search) ? expandExpression(search) : null;

	return function (row) {
		var context = {row: row};
		return expression
			? evalExpression(expression, context)
			: stringContains(row.model.name, search);
	};
}

var EXPR_PREFIX = '{';
var EXPR_SUFFIX = '}';
var EXPR_REGEX = /{.*}/;
var ROW_FIELD_REGEX = /\$/g;

function isExpression(str) {
	return EXPR_REGEX.test(str);
}

function expandExpression(expression) {
	var res = expression
		.replace(EXPR_PREFIX, '')
		.replace(EXPR_SUFFIX, '')
		.replace(ROW_FIELD_REGEX, 'this.row.model.');
	return res;
}

function evalExpression(expression, context) {
	let result = null;

	try {
		result = parse(expression, context);
	} catch (e) {
		return true;
	}

	return result;
}

function stringContains(string, substring) {
	if (typeof string === 'undefined') {
		return true;
	}
	return string.toString()
			.toLowerCase()
			.indexOf(substring.toString().toLowerCase()) !== -1;
}

function parse(expression, context) {
	return function () {
		return eval(expression)
	}.call(context);
}


// MAX ROWS
const maxRowsSelect = document.querySelector('[data-max-rows]');
const maxRows = new MaxRows({maxRows: +maxRowsSelect.value});
maxRowsSelect.addEventListener('input', (event) => {
	console.log('event.target.value:', +event.target.value);
	maxRows.maxRows$.next(+event.target.value);
});


// SORTABLE
const sortable = new Sortable({userSortable: true});


// STICKY ROWS
const stickyRows = new StickyRows();

setTimeout(() => stickyRows.setRows(selectableRows.selectedRows$), 5000);


// SCROLLABLE
const scrollable = new Scrollable();
scrollable.scrollExcessPx$.subscribe(px => {
	console.log('dev-app: scrollable excess:' + px);
	document.querySelector('[scroll-excess-log]').innerHTML = `Scroll Excess: ${px}px`;
});

// INSTANTIATE FTABLE
const el = document.querySelector('[data-container]');

const fTable = new FTable(el, [
	// First plugin to execute — keep in mind order of execution.
	filterable,
	selectableRows,
	columnWidths,
	resizableColumns,
	maxRows,
	sortable,
	new DraggableColumns,
	stickyRows,
	scrollable	// Scrollable should in probably most cases be last in the plugin stack
	// Last plugin to execute — keep in mind order of execution.
]);
fTable.columns$.next(columns);
fTable.rows$.next(rows);

